@@ -2,6 +2,7 @@ from __future__ import absolute_import
 
 import abc
 from copy import deepcopy
+import hashlib
 import os
 import re
 import logging
@@ -37,7 +38,7 @@ CL_BLANK = "data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAA
 URI_SCHEME = "cloudinary"
 API_VERSION = "v1_1"
 
-VERSION = "1.24.0"
+VERSION = "1.25.0"
 
 USER_AGENT = "CloudinaryPython/{} (Python {})".format(VERSION, python_version())
 """ :const: USER_AGENT """
@@ -174,6 +175,8 @@ class Config(BaseConfig):
             cloudinary_url = os.environ.get("CLOUDINARY_URL")
             parsed_url = self._parse_cloudinary_url(cloudinary_url)
             self._setup_from_parsed_url(parsed_url)
+        if not self.signature_algorithm:
+            self.signature_algorithm = utils.SIGNATURE_SHA1
 
     def _config_from_parsed_url(self, parsed_url):
         if not self._is_url_scheme_valid(parsed_url):
@@ -275,7 +278,8 @@ class CloudinaryResource(object):
         return self.get_prep_value() + '#' + self.get_expected_signature()
 
     def get_expected_signature(self):
-        return utils.api_sign_request({"public_id": self.public_id, "version": self.version}, config().api_secret)
+        return utils.api_sign_request({"public_id": self.public_id, "version": self.version}, config().api_secret,
+                                      config().signature_algorithm)
 
     @property
     def url(self):
