@@ -65,6 +65,7 @@ __URL_KEYS = [
     'use_root_path',
     'version',
     'long_url_signature',
+    'signature_algorithm',
 ]
 
 __SIMPLE_UPLOAD_PARAMS = [
@@ -124,16 +125,32 @@ __SERIALIZED_UPLOAD_PARAMS = [
 
 upload_params = __SIMPLE_UPLOAD_PARAMS + __SERIALIZED_UPLOAD_PARAMS
 
+SHORT_URL_SIGNATURE_LENGTH = 8
+LONG_URL_SIGNATURE_LENGTH = 32
 
-def compute_hex_hash(s):
+SIGNATURE_SHA1 = "sha1"
+SIGNATURE_SHA256 = "sha256"
+
+signature_algorithms = {
+    SIGNATURE_SHA1: hashlib.sha1,
+    SIGNATURE_SHA256: hashlib.sha256,
+}
+
+
+def compute_hex_hash(s, algorithm=SIGNATURE_SHA1):
     """
-    Compute hash and convert the result to HEX string
+    Computes string hash using specified algorithm and return HEX string representation of hash.
 
-    :param s: string to process
+    :param s:         String to compute hash for
+    :param algorithm: The name of algorithm to use for computing hash
 
-    :return: HEX string
+    :return: HEX string of computed hash value
     """
-    return hashlib.sha1(to_bytes(s)).hexdigest()
+    try:
+        hash_fn = signature_algorithms[algorithm]
+    except KeyError:
+        raise ValueError('Unsupported hash algorithm: {}'.format(algorithm))
+    return hash_fn(to_bytes(s)).hexdigest()
 
 
 def build_array(arg):
@@ -531,7 +548,7 @@ def process_params(params):
             if isinstance(value, list) or isinstance(value, tuple):
                 value_list = {"{}[{}]".format(key, i): i_value for i, i_value in enumerate(value)}
                 processed_params.update(value_list)
-            elif value:
+            elif value is not None:
                 processed_params[key] = value
     return processed_params
 
@@ -547,18 +564,19 @@ def sign_request(params, options):
     api_secret = options.get("api_secret", cloudinary.config().api_secret)
     if not api_secret:
         raise ValueError("Must supply api_secret")
+    signature_algorithm = options.get("signature_algorithm", cloudinary.config().signature_algorithm)
 
     params = cleanup_params(params)
-    params["signature"] = api_sign_request(params, api_secret)
+    params["signature"] = api_sign_request(params, api_secret, signature_algorithm)
     params["api_key"] = api_key
 
     return params
 
 
-def api_sign_request(params_to_sign, api_secret):
+def api_sign_request(params_to_sign, api_secret, algorithm=SIGNATURE_SHA1):
     params = [(k + "=" + (",".join(v) if isinstance(v, list) else str(v))) for k, v in params_to_sign.items() if v]
     to_sign = "&".join(sorted(params))
-    return compute_hex_hash(to_sign + api_secret)
+    return compute_hex_hash(to_sign + api_secret, algorithm)
 
 
 def breakpoint_settings_mapper(breakpoint_settings):
@@ -716,6 +734,7 @@ def cloudinary_url(source, **options):
     use_root_path = options.pop("use_root_path", cloudinary.config().use_root_path)
     auth_token = options.pop("auth_token", None)
     long_url_signature = options.pop("long_url_signature", cloudinary.config().long_url_signature)
+    signature_algorithm = options.pop("signature_algorithm", cloudinary.config().signature_algorithm)
     if auth_token is not False:
         auth_token = merge(cloudinary.config().auth_token, auth_token)
 
@@ -741,7 +760,15 @@ def cloudinary_url(source, **options):
     signature = None
     if sign_url and not auth_token:
         to_sign = "/".join(__compact([transformation, source_to_sign]))
-        hash_fn, chars_length = (hashlib.sha256, 32) if long_url_signature else (hashlib.sha1, 8)
+        if long_url_signature:
+            # Long signature forces SHA256
+            signature_algorithm = SIGNATURE_SHA256
+            chars_length = LONG_URL_SIGNATURE_LENGTH
+        else:
+            chars_length = SHORT_URL_SIGNATURE_LENGTH
+        if signature_algorithm not in signature_algorithms:
+            raise ValueError("Unsupported signature algorithm '{}'".format(signature_algorithm))
+        hash_fn = signature_algorithms[signature_algorithm]
         signature = "s--" + to_string(
             base64.urlsafe_b64encode(
                 hash_fn(to_bytes(to_sign + api_secret)).digest())[0:chars_length]) + "--"
@@ -777,6 +804,13 @@ def cloudinary_api_url(action='upload', **options):
     return base_api_url([resource_type, action], **options)
 
 
+def cloudinary_api_download_url(action, params, **options):
+    params = params.copy()
+    params["mode"] = "download"
+    cloudinary_params = sign_request(params, options)
+    return cloudinary_api_url(action, **options) + "?" + urlencode(bracketize_seq(cloudinary_params), True)
+
+
 def cloudinary_scaled_url(source, width, transformation, options):
     """
     Generates a cloudinary url scaled to specified width.
@@ -875,11 +909,7 @@ def bracketize_seq(params):
 
 
 def download_archive_url(**options):
-    params = options.copy()
-    params.update(mode="download")
-    cloudinary_params = sign_request(archive_params(**params), options)
-    return cloudinary_api_url("generate_archive", **options) + "?" + \
-        urlencode(bracketize_seq(cloudinary_params), True)
+    return cloudinary_api_download_url(action="generate_archive", params=archive_params(**options), **options)
 
 
 def download_zip_url(**options):
@@ -1037,6 +1067,26 @@ def build_upload_params(**options):
     return params
 
 
+def build_multi_and_sprite_params(**options):
+    """
+    Build params for multi, download_multi, generate_sprite, and download_generated_sprite methods
+    """
+    tag = options.get("tag")
+    urls = options.get("urls")
+    if bool(tag) == bool(urls):
+        raise ValueError("Either 'tag' or 'urls' parameter has to be set but not both")
+    params = {
+        "mode": options.get("mode"),
+        "timestamp": now(),
+        "async": options.get("async"),
+        "notification_url": options.get("notification_url"),
+        "tag": tag,
+        "urls": urls,
+        "transformation": generate_transformation_string(fetch_format=options.get("format"), **options)[0]
+    }
+    return params
+
+
 def __process_text_options(layer, layer_parameter):
     font_family = layer.get("font_family")
     font_size = layer.get("font_size")
@@ -1376,13 +1426,15 @@ def check_property_enabled(f):
     return wrapper
 
 
-def verify_api_response_signature(public_id, version, signature):
+def verify_api_response_signature(public_id, version, signature, algorithm=None):
     """
     Verifies the authenticity of an API response signature
 
     :param public_id: The public id of the asset as returned in the API response
-    :param version: The version of the asset as returned in the API response
+    :param version:   The version of the asset as returned in the API response
     :param signature: Actual signature. Can be retrieved from the X-Cld-Signature header
+    :param algorithm: Name of hashing algorithm to use for calculation of HMACs.
+                      By default uses `cloudinary.config().signature_algorithm`
 
     :return: Boolean result of the validation
     """
@@ -1392,10 +1444,14 @@ def verify_api_response_signature(public_id, version, signature):
     parameters_to_sign = {'public_id': public_id,
                           'version': version}
 
-    return signature == api_sign_request(parameters_to_sign, cloudinary.config().api_secret)
+    return signature == api_sign_request(
+        parameters_to_sign,
+        cloudinary.config().api_secret,
+        algorithm or cloudinary.config().signature_algorithm
+    )
 
 
-def verify_notification_signature(body, timestamp, signature, valid_for=7200):
+def verify_notification_signature(body, timestamp, signature, valid_for=7200, algorithm=None):
     """
     Verifies the authenticity of a notification signature
 
@@ -1403,6 +1459,8 @@ def verify_notification_signature(body, timestamp, signature, valid_for=7200):
     :param timestamp: Unix timestamp. Can be retrieved from the X-Cld-Timestamp header
     :param signature: Actual signature. Can be retrieved from the X-Cld-Signature header
     :param valid_for: The desired time in seconds for considering the request valid
+    :param algorithm: Name of hashing algorithm to use for calculation of HMACs.
+                      By default uses `cloudinary.config().signature_algorithm`
 
     :return: Boolean result of the validation
     """
@@ -1415,7 +1473,9 @@ def verify_notification_signature(body, timestamp, signature, valid_for=7200):
     if not isinstance(body, str):
         raise ValueError('Body should be type of string')
 
-    return signature == compute_hex_hash('{}{}{}'.format(body, timestamp, cloudinary.config().api_secret))
+    return signature == compute_hex_hash(
+        '{}{}{}'.format(body, timestamp, cloudinary.config().api_secret),
+        algorithm or cloudinary.config().signature_algorithm)
 
 
 def get_http_connector(conf, options):
